/*
 * EfficientSPCPerObjectData -- Header
 *
 * Author:  Henrik Zimmer <henrik@zimmer.to>
 *
 * Version: $Revision: 1 $
 * Date:    $date$
 */


#ifndef EFFICIENTSPCPEROBJECTDATA_HH
#define EFFICIENTSPCPEROBJECTDATA_HH

////////////////////////////////////////////////////////


#include <OpenMesh/Core/IO/MeshIO.hh>
#include <OpenMesh/Core/Mesh/TriMesh_ArrayKernelT.hh>
#include <OpenFlipper/common/perObjectData.hh>
#include <ObjectTypes/TriangleMesh/TriangleMesh.hh>
#include "Types.hh"
#include "EfficientSPC.hh"

#include <ACG/Scenegraph/LineNode.hh>
#include <ACG/Scenegraph/MaterialNode.hh>
#include <ACG/Scenegraph/PointNode.hh>
#include <ACG/Geometry/bsp/TriangleBSPT.hh> 

////////////////////////////////////////////////////////


class EfficientSPCPerObjectData : public PerObjectData
{
public:

  EfficientSPCPerObjectData( MeshT& _mesh) : efficientspc_(_mesh) 
  { tet_id_ = -1; chmesh_id_ = -1; chmesh_plc_id_ = -1; rem_chmesh_id_ = -1; bbmesh_id_ = -1; bsp_ = NULL; avgel_ = 0.0; sqrtarea_ = -1.;
    mean_ = -1.0;
    std_ = -1.0;
    var_ = -1.0;
    skew_ = -1.0;
    kurt_ = -1.0;
    remtime_ = -1.0;
    contime_ = -1.0;
    cdttime_ = -1.0;
    inttime_ = -1.0;
    dijtime_ = -1.0;

    kplanesmesh_id_ = -1;

    pn_ = NULL;
    ln_ = NULL;
    ln2_ = NULL;
    ln3_ = NULL;
    mn_ = NULL;
    // make sure vertex normals are computed
    _mesh.update_vertex_normals();

    for( TriMesh::EdgeIter e_it = _mesh.edges_begin(); e_it != _mesh.edges_end(); ++e_it)
      avgel_+= (_mesh.point( _mesh.to_vertex_handle( _mesh.halfedge_handle( e_it, 0)))  - _mesh.point( _mesh.to_vertex_handle( _mesh.halfedge_handle( e_it,1)))).norm();
    avgel_ /= Scalar(_mesh.n_edges());

    sqrtarea_ = 0.;
    for( TriMesh::FaceIter f_it = _mesh.faces_begin(); f_it != _mesh.faces_end(); ++f_it)
    {
      sqrtarea_ += fabs(ACG::EfficientSPC::tri_area( f_it, _mesh));
    }
    sqrtarea_ = sqrt(sqrtarea_);
    std::cerr << __FUNCTION__ << " Using normalization sqrt(surface area) = " << sqrtarea_ << std::endl;

    sp_heh_to_parent_.resize( _mesh.n_vertices(), HEH(-1));
    sp_dist_.resize( _mesh.n_vertices(), FLT_MAX);
  }
  
  virtual
  ~EfficientSPCPerObjectData() 
  {
    if( pn_ != NULL)
      delete pn_;
    if( ln_ != NULL)
      delete ln_;
    if( ln2_ != NULL)
      delete ln2_;
    if( ln3_ != NULL)
      delete ln3_;
    if( mn_ != NULL)
      delete mn_;
  }
  
  ACG::EfficientSPC& efficientspc() { return efficientspc_; }

  int & tet_id() { return tet_id_; }
  int & convex_hull_id() { return chmesh_id_; }
  int & convex_hull_plc_id() { return chmesh_plc_id_; }
  int & rem_convex_hull_id() { return rem_chmesh_id_; }
  int & bbmesh_id() { return bbmesh_id_; }
  int & kplanesmesh_id() { return kplanesmesh_id_; }
  Scalar & avgel() { return avgel_; };
  Scalar & sqrtarea() { return sqrtarea_;}

  OpenMeshTriangleBSPT< TriMesh >* & bsp() { return bsp_; }

  Scalar& mean() { return mean_; }
  Scalar& std() { return std_; }
  Scalar& var() { return var_; }
  Scalar& skew() { return skew_; }
  Scalar& kurt() { return kurt_; }

  Scalar& remtime() { return remtime_; }
  Scalar& contime() { return contime_; }
  Scalar& cdttime() { return cdttime_; }
  Scalar& inttime() { return inttime_; }
  Scalar& dijtime() { return dijtime_; }

  ACG::LeanTetMesh& leanmesh() { return leanmesh_; }

  ACG::SceneGraph::LineNode* ln_;
  ACG::SceneGraph::LineNode* ln2_;
  ACG::SceneGraph::LineNode* ln3_;
  ACG::SceneGraph::MaterialNode* mn_;
  ACG::SceneGraph::PointNode* pn_;


  std::vector< HEH    >& sp_parent() { return sp_heh_to_parent_; }
  std::vector< Scalar >& sp_dist() { return sp_dist_; }
private:
  ACG::EfficientSPC efficientspc_;

  // id of tetmesh
  int tet_id_;
  // id of convex hull mesh corresponding to this mesh
  int chmesh_id_;
  // id of planarized polymesh convex hull mesh corresponding to this mesh
  int chmesh_plc_id_;
  // id of remeshed convex hull
  int rem_chmesh_id_;
  // id of bb mesh
  int bbmesh_id_;
  
  // id of kplanes coarse hull
  int kplanesmesh_id_;

  Scalar sqrtarea_; // normalization factor

  // average edge length
  Scalar avgel_;

  // bsp of convex hull mesh (typically)
  OpenMeshTriangleBSPT< TriMesh > * bsp_;

  Scalar mean_;
  Scalar std_;
  Scalar var_;
  Scalar skew_;
  Scalar kurt_;


  Scalar remtime_;
  Scalar contime_;
  Scalar cdttime_;
  Scalar inttime_;
  Scalar dijtime_;

  ACG::LeanTetMesh leanmesh_;

  // shortest paths (1/vertex)
  std::vector< HEH > sp_heh_to_parent_;
  std::vector< Scalar > sp_dist_;
};


#endif // EFFICIENTSPCPEROBJECTDATA_HH defined

